/**
 * Common types, jQuery API, Form and Data controllers, utility functions.
 * @module data/common
 */
// NOTE: getting jQuery element by dom node has a strange effect inside a Svelte action:
// it doesn't see existing elements, and changes done to the element are not visible to the app.
// Getting element by id seems to fix the problem.
// This maybe caused by 'modal' component moving the element up in dom...
export const SVELTE_DATA_STORE = "svelte_data_store";
export const SVELTE_FORM_STORE = "svelte_form_store";
/** Gets jQuery element by id attribute. */
export function jQueryElemById(id) {
    // eslint-disable-next-line @typescript-eslint/unbound-method, @typescript-eslint/typedef
    const jQuery = window.jQuery;
    if (!jQuery) {
        throw new Error("jQuery is not initialized");
    }
    return jQuery(`#${id}`);
}
/** Gets jQuery element by dom node. */
export function jQueryElem(node) {
    // eslint-disable-next-line @typescript-eslint/unbound-method, @typescript-eslint/typedef
    const jQuery = window.jQuery;
    if (!jQuery) {
        throw new Error("jQuery is not initialized");
    }
    return jQuery(node);
}
/*
            dP   oo dP
            88      88
 dP    dP d8888P dP 88
 88    88   88   88 88
 88.  .88   88   88 88
 `88888P'   dP   dP dP

*/
let unum = 100;
/** Generate an unique number */
export function uid() {
    // const num = new Date().getTime();
    // const num = Math.round(window.performance.now());
    unum = unum + 1;
    return `000000${unum % 1000}`.slice(-3);
}
/** Compare two arrays, two Date objects, or two primitives */
export function equalDataTypes(a1, a2) {
    if (Array.isArray(a1) && Array.isArray(a2)) {
        if (a1.length != a2.length) {
            return false;
        }
        for (let i = 0; i < a1.length; i++) {
            if (a1[i] !== a2[i]) {
                return false;
            }
        }
        return true;
    }
    if (a1 instanceof Date && a2 instanceof Date) {
        return a1.getTime() == a2.getTime();
    }
    return a1 === a2;
}
export function pad(n, size) {
    let str = n.toString();
    while (str.length < size) {
        str = "0" + str;
    }
    return str;
}
/** Determine if the argument is object; not primitive, Array, of function */
function isObject(value) {
    return value !== null && typeof value === "object" && !Array.isArray(value);
}
/** Recursive function to copy only matching fields. */
function copyFields(target, source, logName) {
    for (const key in source) {
        if (Object.prototype.hasOwnProperty.call(target, key)) {
            const to = isObject(target[key]);
            const so = isObject(source[key]);
            if (to) {
                if (so) {
                    // obj <- obj
                    copyFields(target[key], source[key], `${logName}.${key}`);
                }
                else {
                    // obj <- prim|fn|arr
                    console.log(`Setting: '${key}' in ${logName} must be an Object.`);
                }
            }
            else {
                if (so) {
                    // prim|fn|arr <- obj
                    console.log(`Setting: '${key}' in ${logName} can't be an Object.`);
                }
                else {
                    // prim|fn|arr <- prim|fn|arr
                    target[key] = source[key];
                }
            }
        }
        else {
            console.log(`Unrecognized setting: '${key}' in ${logName}.`);
        }
    }
}
// /** Utility type to mark all fields and sub-objects Readonly. */
// export type DeepReadonly<T> = T extends unknown
//     ? { readonly [K in keyof T]: DeepReadonly<T[K]> }
//     : T;
function ensureNumberSettings(jQuery) {
    if (!jQuery.fn.number) {
        jQuery.fn.number = {
            settings: {
                decimal: ".", // TODO: implement decimal
                thousandSeparator: " ",
                listSeparator: ", ",
                moneyPrefix: "$",
                moneySuffix: "",
                moneyPrecision: 2,
            },
        };
    }
}
/** Provides acces to Sematic UI settings for different components. */
export class SettingsHelper {
    constructor(compName) {
        this.compName = compName;
    }
    /** Returns settings object. */
    read() {
        const jQuery = window.jQuery;
        if (!jQuery) {
            throw new Error("jQuery is not initialized.");
        }
        ensureNumberSettings(jQuery);
        const settings = jQuery.fn[this.compName]?.settings;
        if (!settings) {
            throw new Error(`Semantic UI ${this.compName} is not initialized.`);
        }
        return settings;
    }
    /** Copy settings into the global SUI components in jQuery. */
    apply(val) {
        const jQuery = window.jQuery;
        if (!jQuery) {
            throw new Error("jQuery is not initialized.");
        }
        ensureNumberSettings(jQuery);
        const settings = jQuery.fn[this.compName]?.settings;
        if (!settings) {
            throw new Error(`Semantic UI ${this.compName} is not initialized.`);
        }
        copyFields(settings, val, this.compName);
    }
}
/** Copy settings into the global SUI components and number object in jQuery. */
export function applyAllSettings(json) {
    const jQuery = window.jQuery;
    if (!jQuery) {
        throw new Error("jQuery is not initialized.");
    }
    ensureNumberSettings(jQuery);
    for (const name in json) {
        const settings = jQuery.fn[name]?.settings;
        if (!settings) {
            console.log("Ignoring unrecognized Semantic UI component: ${name}");
        }
        else {
            copyFields(settings, json[name], name);
        }
    }
}
// function applyAllSettings<T extends SettingsObject>(json: { [key: string]: T }): void {
